{{
*******************************************************************
*Timer.spin                                                       *
* Kyle Crane                                                      *
*                                                                 *
* Based on original sample code from PE-Labs demo.  This adds     *
* the capability to get real-time updates in a variable for the   *
* seconds count.  This allows other objects to keep track of      *
* time without a function call.                                   *
*                                                                 *
*******************************************************************
}}

DAT
  cog                   byte 0                  'Cog that we launched into
  clkPsFlag             byte 0                  'Pause flag indicates the clock should not update
  days                  long 0                  
  hours                 long 0
  minutes               long 0
  seconds               long 0
  trkSeconds            long 0                  'Continous count of seconds since statup
  tenthSeconds          long 0
  stack                 long 0[40] 
  
PUB StartDefault : success
{{
  Start the clock with all default values at time 0 an no real-time
  trackinig variable.
}}
  return Start(0,0,0,0)


PUB Start(setDay, setHrs, setMins, setSecs) : success
{{
  Fully declared start routine that starts the clock with all parameters.
  Any call that places a zero in the tracking address will cause the code
  to ignore the realtime tracking variable and operate without it.
  Passing an address to any of the previous functions that accept it or
  this function will cause the variable to be updated with an accumulating
  count of seconds since the timer started.
}}

  days       := setDay
  hours      := setHrs
  minutes    := setMins
  seconds    := setSecs
  trkSeconds := seconds
  clkPsFlag  := 0
  
  if not cog
    Stop
    cog := cognew(TimeCount, @stack)+1
    success := cog
  else
    success := 0
    

PUB Stop
  if Cog
    cogstop(Cog~ - 1)
    

PUB Get(dayAdr, hourAdr, minAdr, secAdr) | time
{{
  Get the current time.  Values are loaded into variables at the
  addresses provided to the method parameters.

  Parameters:
    dayAddr  -  day variable address
    hourAddr -  hour variable address 
    minAddr  -  minute variable address
    secAddr  -  secondAddress
}}

  long[dayAdr]  := days
  long[hourAdr] := hours 
  long[minAdr]  := minutes
  long[secAdr]  := seconds

PUB GetDays
  return days

PUB GetHours
  return hours

PUB GetMinutes
  return minutes

PUB GetSeconds
  return seconds  

PUB GetTenthsSeconds
  return tenthSeconds

PUB GetTrkSeconds
  return trkSeconds

PUB GetTrkSecondsAdr(trkAdr)
  long[trkAdr] := @trkSeconds

PUB PauseClk
{{
  Pauses the clock at it's current values.
}}
  clkPsFlag := 1
  
PUB RunClk
{{
  Resumes the clock from it's current values.
}}
  clkPsFlag := 0

  
PRI TimeCount | frq, T

  frq := clkfreq/10
  T  := cnt
  
  repeat 
    T += frq
    waitcnt(T)
    if not clkPsFlag 
      tenthSeconds++
      if tenthSeconds == 10
         tenthSeconds~
         seconds++
         trkSeconds ++   
      if seconds == 60
         seconds~
         minutes++
      if minutes == 60
         minutes~
         hours++
      if hours == 24
         hours~
         days++